#  Copyright 2016 Observable Networks
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from io import BytesIO
from gzip import GzipFile


def gunzip_bytes(gz_bytes, *args, **kwargs):
    with BytesIO(gz_bytes) as fd:
        with GzipFile(fileobj=fd, mode='rb') as gz_fd:
            return gz_fd.read()


def gzip_bytes(raw_bytes, *args, **kwargs):
    with BytesIO() as fd:
        with GzipFile(fileobj=fd, mode='wb') as gz_fd:
            gz_fd.write(raw_bytes)

        return fd.getvalue()
